# dupFinder.py
import os, sys
import hashlib
import subprocess
from subprocess import call
from shutil import rmtree

#Create variable DIR if needing to make path relative to Python script location
DIR = os.path.dirname(os.path.realpath(__file__))
 
def findDup(parentFolder):
    # Dups in format {hash:[names]}
    dups = {}
    for dirName, subdirs, fileList in os.walk(parentFolder):
        for filename in fileList:
            # Get the path to the file
            path = os.path.join(dirName, filename)
            # Calculate hash
            file_hash = hashfile(path)
            # Add or append the file path
            if file_hash in dups:
                dups[file_hash].append(path)
            else:
                dups[file_hash] = [path]
    return dups
 
 
# Joins two dictionaries
def joinDicts(dict1, dict2):
    for key in dict2.keys():
        if key in dict1:
            dict1[key] = dict1[key] + dict2[key]
        else:
            dict1[key] = dict2[key]
 
 
def hashfile(path, blocksize = 65536):
    afile = open(path, 'rb')
    hasher = hashlib.md5()
    buf = afile.read(blocksize)
    while len(buf) > 0:
        hasher.update(buf)
        buf = afile.read(blocksize)
    afile.close()
    return hasher.hexdigest()
 
 
def printResults(dict1):
    results = list(filter(lambda x: len(x) > 1, dict1.values()))
    duplicates = 0
    if len(results) > 0:
        for result in results:
            for subresult in result:
				duplicates = duplicates + 1
				os.remove(subresult)
	print('')
	if duplicates < 192:
		tilesleft = 192-abs(duplicates-192)
		print('Your image has used up '+str(192-duplicates)+' out of the 192 unique 8x8 possible tiles.')
		print('You have '+str(tilesleft)+' tiles worth of wiggle room to spare.')
	elif duplicates == 192:
		print('Your image has used up '+str(192-duplicates)+' out of the 192 unique 8x8 possible tiles.')
		print('Flawless Victory!')
	else:
		print('Unique 8x8 tile total: '+str(duplicates))
		print('WARNING! Your image is over the 192 unique 8x8 tile-limit by '+str(abs(192-duplicates))+' tiles.')
		print('Please consider reducing this number before using with GB Studio.')
 
    else:
        print('No duplicate files found.')
 
 
if __name__ == '__main__':
    if len(sys.argv) > 1:
        dups = {}
        folders = sys.argv[1:]
        tmpdir = ''.join(folders)
        for i in folders:
            # Iterate the folders given
            if os.path.exists(i):
                # Find the duplicated files and append them to the dups
                joinDicts(dups, findDup(i))
            else:
                print('%s is not a valid path, please verify' % i)
                sys.exit()
        printResults(dups)
    else:
        print('Usage: python dupFinder.py folder or python dupFinder.py folder1 folder2')
        
#Cleanup temporary folder
rmtree(tmpdir, ignore_errors=True)
