#!python
'''
    Title: img2gb
    Author: TheOuterLinux
    Purpose: To convert modern image formats -- such as PNG, JPEG, GIF, 
             etc., into Gameboy colors for use with GB Studio.
    
    Last updated: 2020/10/22

    License:
    --------
    Copyright (C) 2020 TheOuterLinux

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

    img2gb.py requirements:
    ----------------------
        ImageMagick
        Python2.7 or Python3
            Modules:
                PIL (Python 2.7 - 3.5 via pip) or python3-pillowfight (Python 3.6 +)
                ptyprocess
                six
                colorama
                
    This script has only been tested on GNU/Linux, but was written in
    a way that should hopefully work just fine on MacOS or Windows.
'''
import os
from os import system
from os import listdir
import sys
import getopt
import signal
import operator
from PIL import Image
import subprocess
from subprocess import call
import tempfile
from tempfile import gettempdir
from shutil import rmtree
from six.moves import input as raw_input
from colorama import init, Fore, Back, Style

import signal
import time
import sys

def clear(): 
    # check and make call for specific operating system 
    _ = call('clear' if os.name =='posix' else 'cls') 

#Filter ANSI escape sequences out of any text sent to stdout or stderr, 
#and replace them with equivalent Win32 calls if using Windows.
if os.name == 'Windows': init()

#Create temporary directory to work in.
#This method makes things much more cross-platform.
tmp = os.path.join(gettempdir(), 'img2gb_{}'.format(hash(os.times())))
os.makedirs(tmp)

#Create variable DIR if needing to make path relative to Python script location
DIR = os.path.dirname(os.path.realpath(__file__))

#Generate background and sprite color palettes
#Background...
cmd = 'convert -size 60x60 xc:"#071821" xc:"#306850" xc:"#86c06c" xc:"#e0f8cf" +append GIF87:' + '"'+tmp+'/BGPAL.gif'+'"'
subprocess.call(cmd, shell=True)
#Sprite...
cmd = 'convert -size 60x60 xc:"#071821" xc:"#86c06c" xc:"#e0f8cf" xc:"#65ff00" +append GIF87:' + '"'+tmp+'/SPRPAL.gif'+'"'
subprocess.call(cmd, shell=True)

#Clear the terminal screen
clear()

#Title
if os.name == 'Windows':
	myCoolTitle='img2gb.py by TheOuterLinux'
	system('title '+myCoolTitle)
else:
	 sys.stdout.write('\x1b]2;img2gb.py by TheOuterLinux\x07')
print (' ' + Fore.GREEN + Back.GREEN + '++------------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' img2gb.py - Convert an image for use with GB Studio              ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)         ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)        ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
#print (' ' + Fore.GREEN + Back.GREEN + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.GREEN + Back.GREEN + "||"+ Fore.BLACK + Back.GREEN + "           Use 'q' or 'quit' any time to end program.             " + Fore.GREEN + Back.GREEN + "||" + Style.RESET_ALL)
print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK)

#Print temporarily created palette locations
print(' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' Temporary folder created in ' + Fore.WHITE + tmp + Fore.YELLOW + '.')
print(' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '')
     
# Ask for the full path of the file to convert. The xfce4-terminal
# browser supports drag-and-drop.
filepath = raw_input(' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' File to convert (full path): ' + Style.RESET_ALL)
filepath = filepath.replace("'", '')
filepath = filepath.strip()
    
if filepath in ['q','quit','Q','Quit','QUIT','Exit','EXIT','exit']:
	rmtree(tmp, ignore_errors=True)
	print('')
	exit()
if os.path.exists(filepath):
	print(' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' ' + Style.RESET_ALL)
else:
	clear()
	print (' ' + Fore.GREEN + Back.GREEN + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' Convert an image for use with GB Studio                          ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)         ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)        ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
	#print (' ' + Fore.GREEN + Back.GREEN + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + "||"+ Fore.BLACK + Back.GREEN + "           Use 'q' or 'quit' any time to end program.             " + Fore.GREEN + Back.GREEN + "||" + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '                                                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' You must input an existing full file path to use this program.     ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' Please use one of the following formats:                           ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' ----------------------------------------                           ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.blp                 *.pcx                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.bmp                 *.pdf                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.cur                 *.png                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.gbr                 *.ppm                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.gif                 *.psd                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.icns                *.tga                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.ico                 *.tif, tiff                              ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.jp2 (JPEG 2000)     *.wal                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.jpg, jpeg           *.webp                                   ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.mic                 *.xbm                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.mps                 *.xpm                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.palm                *.xv                                     ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '                                                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '                                                                      ' + Back.BLACK + ' ')
	print ('' + Style.RESET_ALL)
	rmtree(tmp, ignore_errors=True)
	exit()

# Convert file extensions to all lower-case since older FAT file systems
# tend to use all upper-case while newer file systems can use upper and
# lower-case. Avoids the hassle of writing more if-than statements than
# necessary.
fileext = os.path.splitext(filepath.lower())[1]
extcount = len(fileext)

# Get the file name without the full path or extension (in most cases)
base = os.path.basename(filepath)
base = os.path.splitext(base)[0]

# Get the path of the directory the original file is in so we know where 
# to put the converted result.
dirpath = os.path.dirname(filepath)

# The file name and path to use for a preview file
previewfile = dirpath + '/' + base + '_img2gb.png'

# Prepare the file name to make it DOS-friendly by removing
# spaces and shortening the file name to 8 characters. Using 'qbd' as
# the file extension to stand for 'QuickBASIC DATA'. And because it is
# still plain text, it shouldn't trick the computer into thinking it is
# a Quicken file.
dosbase = base.replace(' ', '')
dosbase = dosbase[:8]
dosbase = dosbase.upper()

# The PIL library supports more formats than this, but only supporting
# the following makes the most sense.
if  fileext in ['.blp', '.bmp', '.cur', '.gbr', '.gif', '.icns', '.ico', \
                '.jp2', '.jpeg', '.jpg', '.mic', '.msp', '.palm', '.pcx', \
                '.pdf', '.png', '.ppm', '.psd', '.tga', '.tif', '.tiff', \
                '.wal', '.webp', '.xbm', '.xpm', '.xv']:
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' File format ' + Fore.WHITE + fileext + Fore.YELLOW + ' supported.')
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '')
else:
	clear()
	print (' ' + Fore.GREEN + Back.GREEN + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' Convert an image for use with GB Studio                          ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)         ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)        ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
	#print (' ' + Fore.GREEN + Back.GREEN + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + "||"+ Fore.BLACK + Back.GREEN + "           Use 'q' or 'quit' any time to end program.             " + Fore.GREEN + Back.GREEN + "||" + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '                                                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	if extcount == 0: print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' You must input an existing full file path to use this program.     ' + Back.GREEN + ' ' + Style.RESET_ALL)
	if extcount == 1: print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                      ' + Back.GREEN + ' ' + Style.RESET_ALL)
	if extcount == 2: print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                     ' + Back.GREEN + ' ' + Style.RESET_ALL)
	if extcount == 3: print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	if extcount == 4: print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                   ' + Back.GREEN + ' ' + Style.RESET_ALL)
	if extcount == 5: print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                  ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' Please use one of the following formats:                           ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' ----------------------------------------                           ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.blp                 *.pcx                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.bmp                 *.pdf                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.cur                 *.png                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.gbr                 *.ppm                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.gif                 *.psd                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.icns                *.tga                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.ico                 *.tif, tiff                              ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.jp2 (JPEG 2000)     *.wal                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.jpg, jpeg           *.webp                                   ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.mic                 *.xbm                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.mps                 *.xpm                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '     *.palm                *.xv                                     ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '                                                                    ' + Back.GREEN + ' ' + Style.RESET_ALL)
	print (' ' + Fore.GREEN + Back.GREEN + '                                                                      ' + Back.BLACK + ' ')
	print ('' + Style.RESET_ALL)
	rmtree(tmp, ignore_errors=True)
	exit()

# Get the selected image file's width and height
im = Image.open(filepath)
width, height = im.size
colorcount = len(list(filter(None, im.histogram())))

# Ask what MODE will be used
# Modes commented-out are not yet supported but may be one day and I'm
# too lazy to look-up their specs again.
print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '')
print (' ' + Fore.WHITE + Back.GREEN + ' ' + Back.BLACK + ' Options:')
print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' ------')
print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' Logo or Title Screen ' + Fore.WHITE + '1' + Fore.YELLOW + ' - 160x144 maximum')
print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' Background ' + Fore.WHITE + '2' + Fore.YELLOW + ' - 256x256 maximum')
print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' Sprite ' + Fore.WHITE + '3' + Fore.YELLOW + ' - 400x16 maximum; 25 frames for animation')
print (' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + '')
screenmode = raw_input(' ' + Fore.GREEN + Back.GREEN + ' ' + Back.BLACK + ' What ' + Fore.WHITE + 'option' + Fore.YELLOW + ' do you plan on using? (' + Fore.WHITE + '1,2,3..,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
screenmode = screenmode.lower()
clear()

print(' ' + Fore.GREEN + Back.GREEN + '+-----------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' img2gb.py - Convert an image for use with GB Studio            ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
print(' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)       ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
print(' ' + Fore.GREEN + Back.GREEN + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)      ' + Fore.GREEN + Back.GREEN + '||' + Style.RESET_ALL)
#print (' ' + Fore.GREEN + Back.GREEN + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
print(' ' + Fore.GREEN + Back.GREEN + "||"+ Fore.BLACK + Back.GREEN + "           Use 'q' or 'quit' any time to end program.           " + Fore.GREEN + Back.GREEN + "||" + Style.RESET_ALL)
print('')
print(Style.RESET_ALL)
def screen():
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + ' It seems as though your image is quite large (' + Fore.RED + '> ' + str(screenw) + 'x' + str(screenh) + Fore.WHITE + ').       ' + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + ' If you plan on creating with ' + Fore.YELLOW + 'GB Studio' + Fore.WHITE + ' and then later opening   ' + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + " with " + Fore.YELLOW + "an emulator" + Fore.WHITE + ", things will not look right.                 " + Fore.RED + "  " + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + ' So, it is ' + Fore.RED + 'suggested that '+ Fore.RED + 'you shrink the file ' + Fore.WHITE + 'and run this script'+ Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + ' again. However, who am I to stop you.                           ' + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + '                                                                 ' + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + ' Stretching in the following case refers                         ' + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + ' to ' + Fore.RED + 'forcing' + Fore.WHITE + ' a width and height of ' + Fore.RED + screendim + Fore.WHITE + ' as opposed to keeping  ' + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + ' ' + Back.BLACK + Fore.WHITE + ' the original aspect ratio of the image.                         ' + Back.GREEN + ' ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.GREEN + '                                                                   ' + Back.MAGENTA + ' ' + Style.RESET_ALL)
	    print (' ' + Back.BLACK + ' ' + Back.MAGENTA + '                                                                   ' + Style.RESET_ALL)
	    print('')

#Logo or Title screen
if screenmode in ['1','logo','title']:
	screenmode = '1'
	print (Style.RESET_ALL)
	print (' ' + Fore.BLACK + Back.GREEN +' ' + Back.WHITE + ' Logo or Title selected...  ' + Style.RESET_ALL)
	print (' ' + Back.GREEN + '                                                                   ' + Style.RESET_ALL)
	colornumber = 4
	screenw = 160
	screenh = 144
	screendim = str(screenw) + 'x' + str(screenh)
	width = int( 8 * round( width / 8. ))
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#Background
elif screenmode in ['2','background']:
	screenmode = '2'
	print (Style.RESET_ALL)
	print (' ' + Fore.BLACK + Back.GREEN +' ' + Back.WHITE + ' Background selected...  ' + Style.RESET_ALL)
	print (' ' + Back.GREEN + '                                                                   ' + Style.RESET_ALL)
	colornumber = 4
	screenw = 256
	screenh = 256
	screendim = str(screenw) + 'x' + str(screenh)
	width = int( 8 * round( width / 8. ))
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#Sprite
elif screenmode in ['4','sprite']:
	screenmode = '3'
	print (Style.RESET_ALL)
	print (' ' + Fore.BLACK + Back.GREEN +' ' + Back.WHITE + ' Sprite selected...  ' + Style.RESET_ALL)
	print (' ' + Back.GREEN + '                                                                   ' + Style.RESET_ALL)
	colornumber = 4
	screenw = 400
	screenh = 16
	screendim = str(screenw) + 'x' + str(screenh)
	width = int( 8 * round( width / 8. ))
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#QUIT
elif screenmode in ['q','quit','Q','Quit','QUIT','Exit','EXIT','exit']: exit()

#If invalid option picked, choose Sprite
else:
	print (Style.RESET_ALL)
	print (' ' + Fore.BLACK + Back.GREEN +' ' + Back.WHITE + ' Sprite selected...  ' + Style.RESET_ALL)
	print (' ' + Back.GREEN + '                                                                   ' + Style.RESET_ALL)
	colornumber = 4
	screenw = 400
	screenh = 16
	screendim = str(screenw) + 'x' + str(screenh)
	width = int( 8 * round( width / 8. ))
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

clear()

negate=""
def previewmaker():
	if screenmode in ['1','logo','title']:
		if colorcount > colornumber:
			cmd = 'convert ' + dimensions + ' -alpha off -fill "#071821" -colors 4 -remap '+'"'+tmp+'/BGPAL.gif" +antialias ' + '"'+filepath+'" ' + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
		else:
			cmd = 'convert ' + dimensions + ' -alpha off -fill "#071821" -colors 4 -remap '+'"'+tmp+'/BGPAL.gif" +dither +antialias ' + '"'+filepath+'" ' + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
	elif screenmode in ['2','background']:
		if colorcount > colornumber:
			cmd = 'convert ' + dimensions + ' -alpha off -fill "#071821" -colors 4 -remap '+'"'+tmp+'/BGPAL.gif" +antialias ' + '"'+filepath+'" ' + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
		else:
			cmd = 'convert ' + dimensions + ' -alpha off -fill "#071821" -colors 4 -remap '+'"'+tmp+'/BGPAL.gif" +dither +antialias ' + '"'+filepath+'" ' + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
	else:
		if  colorcount > colornumber:
			cmd = 'convert ' + dimensions + ' -alpha off -fill "#65ff00" -colors 4 -remap '+'"'+tmp+'/SPRPAL.gif" +antialias ' + '"'+filepath+'" ' + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
		else:
			cmd = 'convert ' + dimensions + ' -alpha off -fill "#65ff00" -colors 4 -remap '+'"'+tmp+'/SPRPAL.gif" +dither +antialias ' + '"'+filepath+'" ' + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)

#Create converted image...
previewmaker()

print('Your converted image can now be found in...')
print(previewfile)
print('')

#Take converted image and then split it into several 8x8 pieces...
print('Now we are creating and scanning '+str(tmp)+' for unique 8x8 tiles...')
print('Please be patient.')
cmd = 'convert ' + '"'+previewfile+'"' + ' -crop 8x8 -background black -extent 8x8 +repage -scene 1 GIF87:'+tmp+'/split%03d.gif'
subprocess.call(cmd, shell=True)

# Do a FOR-LOOP to create an 8x8 array for each 8x8 image within the
# temporary directory.
def makearray():
	im = Image.open(tmp + '/' + filename)
	rgb_im = im.convert('RGB')
	width, height = im.size
	pixelx = 1
	pixely = 1
	with open(tmp + '/' + dosbase + '_' + str(filenumber) + '.txt', "w+") as outfile:
		for pixely in range(height):
			row = []
			for pixelx in range(width):
				r,g,b = rgb_im.getpixel((pixelx, pixely))
				if [r, g, b] == [101, 255, 0]: row.append("0") #Transparent sprite color
				elif [r, g, b] == [7, 24, 33]: row.append("1") #Dark green
				elif [r, g, b] == [134, 192, 108]: row.append("2") #Shamrock green
				elif [r, g, b] == [224, 248, 207]: row.append("3") #Pale green
				elif [r, g, b] == [48, 104, 80]: row.append("4") #Pine green
				else: row.append("0")
			outfile.write(",".join(row) + '\n')

filenumber = 1
filelist = sorted(os.listdir(tmp)) #Process files in alphabetical order
for filename in filelist:
	filenumber = filenumber + 1
	makearray()
	
for file_name in listdir(tmp):
    #4
    if file_name.endswith('.gif'):
        #5
        os.remove(tmp+'/'+file_name)

# The Python script that does the unique 8x8 tile counting
cmd = ('python '+DIR+'/Tools/same.py '+str(tmp))
subprocess.call(cmd, shell=True)


# Cleanup; deletes the temporary directory this script created at the beginning
#rmtree(tmp, ignore_errors=True)

#Return colors back to normal on Windows
if os.name == 'Windows': deinit()
